(ns coderloop.lying
  (:use [clojure.contrib
	 [duck-streams :only [read-lines]]
	 [seq :only [find-first]]
	 [str-utils :only [re-gsub]]])
  (:use rlm.shell-inspect)
  (:use rlm.map-utils)
  (:use [clojure [string :only [split trim]]]))



(comment
6
JohnDoe   -  1
MisterBlanko
JohnTitor -  2
DirkWhatever
BrunoJoeConner
DirkWhatever -  1
FooBar
BrunoJoeConner -  2
JohnTitor
MisterBlanko
MisterBlanko   -    1
DirkWhatever
FooBar - 3
DirkWhatever
BrunoJoeConner
JohnDoe
)


(def witnesses
     {:john-doe         [:mister-blanco]
      :john-tator       [:dirk-whatever :bruno-joe-conner]
      :dirk-whatever    [:foo-bar]
      :bruno-joe-conner [:john-tator :mister-blanco]
      :mister-blanco    [:dirk-whatever]
      :foo-bar          [:dirk-whatever :bruno-joe-conner :john-doe]})

(def t "/home/r/coderloop-test/test.txt")
(def a "/home/r/coderloop-test/cluedo-a.in")

(defn parse-file [f]
  (let [lines (map clojure.string/trim (rest (read-lines f)))]
    (map-keys
     (comp (partial re-gsub #"\W*:\W*\d*\W*" "") first)
     (apply hash-map
	    (partition-by #(re-matches #".*:.*" %) lines)))))
     

(defrecord verdict [good bad])

(def initial-verdict (verdict. #{} #{}))

(defn valid-verdict? [num verdict]
  (and
   (>= num (+ (count (:good verdict)) (count (:bad verdict))))
   (not (reduce #(or %1 %2)
	   (map (:good verdict) (:bad verdict))))))

(defn expand-verdict [judgements [person accuse]]
  (let [y (flatten
	   (for [record judgements]
	     [(verdict. (into (:good record) (vector person))
			(into (:bad  record) accuse))
	      (verdict. (into (:good record) accuse)
			(into (:bad  record) (vector person)))]))]
;;    (println (count y))
;;    (dorun (map println y))
    y))

(defn determine-guilt [witnesses]
  (let [valid? (partial valid-verdict? (count witnesses))]
    (filter valid?
	    (reduce (comp
		     (partial filter valid?)
		     expand-verdict)
		    [initial-verdict] witnesses))))

(defn print-soln [witnesses]
  (let [verdict  (first (determine-guilt witnesses))
	group-nums (sort [(count (:good verdict)) (count (:bad verdict))])]
    (println (str (first group-nums) \: (second group-nums)))))


(if (command-line?)
  (print-soln (parse-file (first *command-line-args*))))
